#!/usr/bin/python3

import unittest
import os
import sys
import logging
#logging.basicConfig(level=logging.INFO)

# Define a helper function to make a unicode string in v2 and v3
import sys
if sys.version < '3':
    import codecs
    def u(x):
        return codecs.unicode_escape_decode(x)[0]
else:
    def u(x):
        return x

# Add parent to the start of os.path, import ucammap
sys.path.insert(0,os.path.join(os.path.dirname(__file__),"..")) 
import ucammap

# Note that this test is fragile since it relies on network
# connections and the Map API server

class APITestCase(unittest.TestCase):

    def setUp(self):
        self.api = ucammap.API()

    def testType(self):
        self.assertTrue(isinstance(self.api,ucammap.API))

    def testSearch(self):
        """Basic search test"""

        result = self.api.search(q = 'computing')

        self.assertTrue(isinstance(result,list))
        self.assertTrue(len(result) > 0)
        self.assertTrue(isinstance(result[0],ucammap.Entity))

    def testUnicode(self):
        """Unicode character test"""

        result = self.api.search(inst = 'moller')

        self.assertTrue(isinstance(result,list))
        self.assertEqual(len(result), 1)
        self.assertTrue(isinstance(result[0],ucammap.Entity))
        self.assertEqual(result[0].name,u('M\xf8ller Centre'))

#    @unittest.skip("Inflating insts only works for 'parents', and none defined"#)
#    def testInflateInst(self):
#        """Test the inflate method on an inst"""
#
#        inst_data = {
#            "type": "institution",
#            "inst": "cs",
#            "prefix": "University",
#            "name": "Computing Service",
#            "sd": "ucs",
#            "role": "nonacademic"
#            }
#        
#        inst = ucammap.Entity(inst_data)
#        result = self.api.inflate(inst)
#
#        self.assertTrue(isinstance(result,ucammap.Entity))
#
#        self.assertEqual(inst.name,result.name)
#        self.assertEqual(inst.parents,[])
#        self.assertNotEqual(result.parents,[])

    def testInflateRef(self):
        """Test the inflate method on someting with a ref"""

        building_data = {
            "type": "building",
            "ref": "M039",
            "name": "Cockcroft Building",
            "lat": 52.203322,
            "lon": 0.119814,
            "id": 95412331
            }
        
        building = ucammap.Entity(building_data)
        result = self.api.inflate(building)

        self.assertTrue(isinstance(result,ucammap.Entity))

        self.assertEqual(building.name,result.name)
        self.assertEqual(building.entrances,[])
        self.assertNotEqual(result.entrances,[])


    def testInflateErrors(self):
        """Test inflate error handling"""
        
        
        nonsuch_data = {
            "type": "building",
            "ref": "zzzz",
            }
        nonsuch = ucammap.Entity(nonsuch_data)
        result = self.api.inflate(nonsuch)
        self.assertTrue(isinstance(result,ucammap.Entity))
        self.assertEqual(nonsuch,result)

        self.assertRaises(TypeError, self.api.inflate, 0)



if __name__ == '__main__':
    unittest.main()
